#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>

#define PodType POD_TYPE_ALARM
#define POD_VAR_URL "http://pel.cs.byu.edu/~sorenson/Thesis/PODs/Alarm/"

#include "PODDefines.h"
#include "PODIncludes.h"

#include "Serial.h"

char MorseSpeed;
char MorseTone;
unsigned char MorseBuffer[100];

char AlarmTone;
unsigned char AlarmType;
char AlarmSounding;

int Delay(unsigned int Duration)
{
   unsigned long Counter1, Counter2;
   for (Counter1 = 0 ; Counter1 < Duration ; Counter1 ++)
   {
      for (Counter2 = 0 ; Counter2 < 1000 ; Counter2 ++) ;
   }
   return 0;
}

int Sound(int Duration, int Bit)
{
   sbi(PORTC, Bit);
   Delay(Duration*2);
   cbi(PORTC, Bit);
   return(0);
}

void DiDah(char *String)
{
   unsigned int CharCounter;

   for (CharCounter = 0 ; CharCounter < strlen(String) ; CharCounter ++)
   {
      if (String[CharCounter] == '.') Sound(10,2);
      else Sound(30,2);
      Delay(1);
   }
   Delay(2);
}

void PlayMorse(char Character)
{
   switch (Character)
   {
      case 'a':
         DiDah(".-");
         break;
      case 'b':
         DiDah("-...");
         break;
      case 'c':
         DiDah("-.-.");
         break;
      case 'd':
         DiDah("-..");
         break;
      case 'e':
         DiDah(".");
         break;
      case 'f':
         DiDah("..-.");
         break;
      case 'g':
         DiDah("--.");
         break;
      case 'h':
         DiDah("....");
         break;
      case 'i':
         DiDah("..");
         break;
      case 'j':
         DiDah(".---");
         break;
      case 'k':
         DiDah("-.-");
         break;
      case 'l':
         DiDah(".-..");
         break;
      case 'm':
         DiDah("--");
         break;
      case 'n':
         DiDah("-.");
         break;
      case 'o':
         DiDah("---");
         break;
      case 'p':
         DiDah(".--.");
         break;
      case 'q':
         DiDah("--.-");
         break;
      case 'r':
         DiDah(".-.");
         break;
      case 's':
         DiDah("...");
         break;
      case 't':
         DiDah("-");
         break;
      case 'u':
         DiDah("..-");
         break;
      case 'v':
         DiDah("...-");
         break;
      case 'w':
         DiDah(".--");
         break;
      case 'x':
         DiDah("-..-");
         break;
      case 'y':
         DiDah("-.--");
         break;
      case 'z':
         DiDah("--..");
         break;
      case '0':
         DiDah("-----");
         break;
      case '1':
         DiDah(".----");
         break;
      case '2':
         DiDah("..---");
         break;
      case '3':
         DiDah("...--");
         break;
      case '4':
         DiDah("....-");
         break;
      case '5':
         DiDah(".....");
         break;
      case '6':
         DiDah("-....");
         break;
      case '7':
         DiDah("--...");
         break;
      case '8':
         DiDah("---..");
         break;
      case '9':
         DiDah("----.");
         break;
      case '.':
         DiDah(".-.-.-");
         break;
      case ',':
         DiDah("--..--");
         break;
      case '?':
         DiDah("..--..");
         break;
      case ' ':
         Delay(4);
         break;
      default:
         UART_TransmitLine("Unknown character: ");
         UART_TransmitByte(Character);
         UART_TransmitLine("\n\r");
         break;
   }
}

int Init()
{
   outp(0x07, DDRC);
   outp(0x00, PORTC);
   MorseBuffer[0] = 0; /* buffer length is stored in 0 */
   AlarmSounding = 0;

   UART_Init(23);
   sei();
   return 0;
}

void Process_Command_Set_Morse_Speed()
{
   MorseSpeed = UART_ReceiveByte();
   UART_TransmitByte(POD_RESPONSE_ACK);
}

void Process_Command_Set_Morse_Tone()
{
   MorseTone = UART_ReceiveByte();
   UART_TransmitByte(POD_RESPONSE_ACK);
}

unsigned char DeQueueMorseChar()
{
   unsigned char Counter;
   unsigned char ReturnChar;

   if (MorseBuffer[0] == 0) return(0);
   ReturnChar = MorseBuffer[1];
   for (Counter = 1 ; Counter < MorseBuffer[0] ; Counter ++)
   {
      MorseBuffer[Counter] = MorseBuffer[Counter + 1];
   }
   MorseBuffer[0] --;
   return(ReturnChar);
}

void QueueMorseChar(unsigned char NewChar)
{
   MorseBuffer[++MorseBuffer[0]] = NewChar;
}

char MorseWaiting()
{
   return(MorseBuffer[0]);
}

void Process_Command_Play_Morse()
{
   unsigned char Counter;
   unsigned char NewCharacterCount;

   NewCharacterCount = UART_ReceiveByte();
   for (Counter = 0 ; Counter < NewCharacterCount ; Counter ++)
   {
      QueueMorseChar(UART_ReceiveByte());
   }
   UART_TransmitByte(POD_RESPONSE_ACK);
}

void Process_Command_Quiet()
{
   while (MorseBuffer[0] > 0) /* flush the Morse Code buffer */
   {
      DeQueueMorseChar();
   }
   AlarmSounding = 0; /* Turn off any alarms */
   UART_TransmitByte(POD_RESPONSE_ACK);
}

void Process_Command_Sound()
{
   unsigned char Duration;

   Duration = UART_ReceiveByte();
   UART_TransmitByte(POD_RESPONSE_ACK);
   Sound(Duration, 2);
}

int Process_Command(unsigned int Command)
{
   switch (Command)
   {
      case POD_COMMAND_PING:
         Process_Command_Ping();
         break; 
      case POD_COMMAND_GETTYPE:
         Process_Command_GetType();
         break;
      case POD_COMMAND_GETINFO:
         Process_Command_GetInfo();
         break;
//      case POD_COMMAND_RESET:
//         Process_Command_Reset();
//         break;
      case POD_COMMAND_GETNAME:
         Process_Command_GetName();
         break;
      case POD_COMMAND_SETNAME:
         Process_Command_SetName();
         break;
      case POD_COMMAND_ALARM_SET_MORSE_SPEED:
         Process_Command_Set_Morse_Speed();
         break;
      case POD_COMMAND_ALARM_SET_MORSE_TONE:
         Process_Command_Set_Morse_Tone();
         break;
      case POD_COMMAND_ALARM_PLAY_MORSE:
         Process_Command_Play_Morse();
         break;
      case POD_COMMAND_ALARM_QUIET:
         Process_Command_Quiet();
         break;
      case POD_COMMAND_ALARM_SOUND:
         Process_Command_Sound();
         break;
      default:
         Send_NACK();
         break;
   }
   return(0);
}


int main()
{
   unsigned int Command;

   Init();

   while (1)
   {
      if (UART_Data())
      {
         Command = UART_ReceiveWord();
         Process_Command(Command);
      }
      if (MorseWaiting()) /* if there's Morse in the buffer */
      {
         PlayMorse(DeQueueMorseChar());
         continue;
      }
      if (AlarmSounding)
      {
      }
   }
}
