#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>

#define PodType POD_TYPE_BUTTONS
#define POD_VAR_URL "http://pel.cs.byu.edu/~sorenson/Thesis/PODs/Buttons/"

#include "PODDefines.h"
#include "PODIncludes.h"

#include "Serial.h"

int Init()
{
   outb(DDRB, 0x00);
   outb(DDRC, 0x00); /* all inputs */
   outb(DDRD, 0x00);

   outb(PORTC, 0x0F); /* turn on pull-up resistors on the pins we care about */

   UART_Init(23);
   sei();
   return 0;
}

void Process_Command_Get_Status()
{
   UART_TransmitByte(POD_RESPONSE_ACK);
   UART_TransmitByte(~(inb(PINC)) & 0x0F);
}

void Process_Command_Get_Number_of_Buttons()
{
   UART_TransmitByte(POD_RESPONSE_ACK);
   UART_TransmitByte(4);
}

int Process_Command(unsigned int Command)
{
   switch (Command)
   {
      case POD_COMMAND_PING:
         Process_Command_Ping();
         break; 
      case POD_COMMAND_GETTYPE:
         Process_Command_GetType();
         break;
      case POD_COMMAND_GETINFO:
         Process_Command_GetInfo();
         break;
//      case POD_COMMAND_RESET:
//         Process_Command_Reset();
//         break;
      case POD_COMMAND_GETNAME:
         Process_Command_GetName();
         break;
      case POD_COMMAND_SETNAME:
         Process_Command_SetName();
         break;
      case POD_COMMAND_BUTTONS_GET_STATUS:
         Process_Command_Get_Status();
         break;
      case POD_COMMAND_BUTTONS_GET_NUMBER:
         Process_Command_Get_Number_of_Buttons();
         break;
      default:
         Send_NACK();
         break;
   }
   return(0);
}


int main()
{
   unsigned int Command;

   Init();

   while (1)
   {
      if (UART_Data())
      {
         Command = UART_ReceiveWord();
         Process_Command(Command);
      }
   }
}
