#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include <string.h>
#include <avr/twi.h>


#define PodType POD_TYPE_COMPASS
#define POD_VAR_URL "http://pel.cs.byu.edu/~sorenson/Thesis/PODs/Compass/"

#include "PODDefines.h"
#include "PODIncludes.h"

#include "Serial.h"
#include "Delay.h"
#include "TWI.h"

#define COMPASS_ADDRESS 0xC0

//#define SYSCLK 14745600UL
#define SYSCLK   3686400UL
//#define SYSCLK   8000000UL

char GetCompassData(unsigned char *Data)
{
   char Result;

   Result = TWI_GetData(COMPASS_ADDRESS, 0, Data, 16);
   return(Result);
}

void Init()
{
   outb(DDRB, 0);
   outb(DDRC, 0);
   outb(DDRD, 0);

   delay(0x0100);

   UART_Init(23);
   TWI_Init(SYSCLK);
   sei();
}

void Process_Command_GetHeading()
{
   char error1;
   unsigned char RawData[16];
   unsigned int Heading;

   error1 = GetCompassData(RawData);
   if (error1 == 0)
   {
      Heading = (RawData[2] << 8) + RawData[3];
      UART_TransmitByte(POD_RESPONSE_ACK);
      UART_TransmitWord(Heading);
   }
   else 
   {
      Send_NACK();
      UART_TransmitByte(0 - error1);
      UART_TransmitByte(0x00);
   }
}

int Process_Command(unsigned int Command)
{
   switch (Command)
   {
      case POD_COMMAND_PING:
         Process_Command_Ping();
         break; 
      case POD_COMMAND_GETTYPE:
         Process_Command_GetType();
         break;
      case POD_COMMAND_GETINFO:
         Process_Command_GetInfo();
         break;
//      case POD_COMMAND_RESET:
//      /*         Process_Command_Reset(); */
//      //         break;
      case POD_COMMAND_GETNAME:
         Process_Command_GetName();
         break;
      case POD_COMMAND_SETNAME:
         Process_Command_SetName();
         break;
      case POD_COMMAND_COMPASS_GET_HEADING:
         Process_Command_GetHeading();
         break;
      default:
         Send_NACK();
         break;
   }
   return(0);
}

int main(void)
{
   unsigned int Command;

   Init();

   while (1)
   {
      if (UART_Data())
      {
         Command = UART_ReceiveWord();
         Process_Command(Command);
      } /* if UART_Data */
   }
}

