#ifndef __ADC_H__
#define __ADC_H__

static unsigned char ADCReady;

SIGNAL(SIG_ADC)
{
   ADCReady = 1;
}

/*
   if Reference = 1, internal reference is used
   Reference = 0, external reference
   Reference = 2, AVcc
*/
int SetupADC(int Channel, int Reference)
{
//   outp(Channel, ADMUX); /* enable analog channel 0 */
   
   if (Reference == 1) /* set the reference and the channel */
   {
      outp(0xC0 | (Channel & 0x0F), ADMUX);
   }
   else if (Reference == 2) /* AVcc */
   {
      outp(0x40 | (Channel & 0x0F), ADMUX);
   }

   outp(0x88 | 0x07, ADCSR); /* ADC enable and set prescaler */
   sbi(ADCSR, 6); /* trigger dummy read */
   sei();
   return(0);
}

int StartADC(int Channel)
{
   int Result;

   ADCReady = 0;
   sbi(ADCSR, 6);

   SleepMode(1);
   Result = inp(ADCL);
   Result += inp(ADCH) << 8;
   return(Result);
}

unsigned long ADC_GetReading()
{
   unsigned long ADCValue;
   unsigned long ADCAccumulation;
   int Counter;

   ADCAccumulation = 0;
   for (Counter = 0 ; Counter < 16 ; Counter ++)
   {
      ADCValue = StartADC(0);
      ADCAccumulation += ADCValue;
   }
   ADCValue = ADCAccumulation / 16;
   return(ADCValue);
}

#endif
