#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>

#define PodType POD_TYPE_POWERSENSOR
#define POD_VAR_URL "http://pel.cs.byu.edu/~sorenson/Thesis/PODs/PowerSensor/"

#include "PODDefines.h"
#include "PODIncludes.h"

#include "Serial.h"
#include "Delay.h"

void Lights(char Setting)
{
   outb(PORTC, Setting);
}

char GetPowerStatus()
{
   unsigned char Data;

   Data = inb(PINB);
   if (Data & 0x01) return(0);
   return(1);
}

char Init()
{
   UART_Init(23);
   outb(DDRB, 0x00);
   outb(DDRC, 0x03);
   sei();
   return(0);
}

void Process_Command_GetStatus()
{
   char Status;
   
   Status = GetPowerStatus();
   UART_TransmitByte(POD_RESPONSE_ACK);
   UART_TransmitByte(Status);
}

int Process_Command(unsigned int Command)
{
   switch (Command)
   {
      case POD_COMMAND_PING:
         Process_Command_Ping();
         break; 
      case POD_COMMAND_GETTYPE:
         Process_Command_GetType();
         break;
      case POD_COMMAND_GETINFO:
         Process_Command_GetInfo();
         break;
//      case POD_COMMAND_RESET:
/*         Process_Command_Reset(); */
//         break;
      case POD_COMMAND_GETNAME:
         Process_Command_GetName();
         break;
      case POD_COMMAND_SETNAME:
         Process_Command_SetName();
         break;
      case POD_COMMAND_POWERSENSOR_GET_STATUS:
         Process_Command_GetStatus();
         break;
      default:
         Send_NACK();
         break;
   }
   return(0);
}

int main()
{
   int Status, PreviousStatus;
   unsigned int Command;
   unsigned char TempChar;
   Init();

   Status = GetPowerStatus();
   if (Status) Lights(2);
   else Lights(1);
   while (1)
   {
      Status = GetPowerStatus();
      if (Status) Lights(2);
      else Lights(1);
      if (UART_Data()) /* some data is waiting */
      {
         Command = UART_ReceiveWord();
         Process_Command(Command);
      } /* if UART_Data */
   }
}
