#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include <string.h>
#include <avr/twi.h>

#define PodType POD_TYPE_DISTANCE
#define POD_VAR_URL "http://pel.cs.byu.edu/~sorenson/Thesis/PODs/Sonar/"

#include "PODDefines.h"
#include "PODIncludes.h"

#include "Serial.h"
#include "Delay.h"
#include "TWI.h"

#define SONAR_ADDRESS 0xE0

#define SONAR_COMMAND_REGISTER 0
#define SONAR_COMMAND_INCHES 0x50 //result in inches
#define SONAR_COMMAND_CM     0x51 //result in cm
#define SONAR_COMMAND_USEC   0x52 //result in usec

#define SONAR_MAX_ADDRESS 35 //last byte available (36 bytes 0:35)
//#define SYSCLK 14745600UL
#define SYSCLK   3686400UL
//#define SYSCLK   8000000UL


char SonarStart(unsigned char Command)
{
   unsigned char Data[2];
   char Result;

   Data[0] = Command;
   
   Result = TWI_SendCommand(SONAR_ADDRESS, SONAR_COMMAND_REGISTER, Data, 1);
   if (Result) return Result;
   delay(0x0200);

   return 0;
}

char GetData(unsigned char *Data)
{
   char Result;

   Result = TWI_GetData(SONAR_ADDRESS, 0, Data, 0x24);
   if (Result) return -1;
   return 0;
}

void Init()
{
   outb(DDRB, 0);
   outb(DDRC, 0);
   outb(DDRD, 0);

   delay(0x0100);

   UART_Init(23);
   TWI_Init(SYSCLK);
   sei();
}

void Process_Command_GetDistance(unsigned char Command)
{
   char error1;
   unsigned char RawData[36];
   unsigned int Distance;

   error1 = SonarStart(Command);
   if (!error1)
   {
      error1 = GetData(RawData);
      if (!error1)
      {
         Distance = (RawData[2] << 8) | RawData[3];
         UART_TransmitByte(POD_RESPONSE_ACK);
         UART_TransmitWord(Distance);
      }
   }
   if (error1)
   {
      Send_NACK();
      UART_TransmitByte(error1);
      UART_TransmitByte(0x00);
   }
}

int Process_Command(unsigned int Command)
{
   switch (Command)
   {
      case POD_COMMAND_PING:
         Process_Command_Ping();
         break; 
      case POD_COMMAND_GETTYPE:
         Process_Command_GetType();
         break;
      case POD_COMMAND_GETINFO:
         Process_Command_GetInfo();
         break;
//      case POD_COMMAND_RESET:
//         Process_Command_Reset();
//         break;
      case POD_COMMAND_GETNAME:
         Process_Command_GetName();
         break;
      case POD_COMMAND_SETNAME:
         Process_Command_SetName();
         break;
      case POD_COMMAND_DISTANCE_GET_INCHES:
         Process_Command_GetDistance(SONAR_COMMAND_INCHES);
         break;
      case POD_COMMAND_DISTANCE_GET_CM:
         Process_Command_GetDistance(SONAR_COMMAND_CM);
         break;
      case POD_COMMAND_DISTANCE_GET_USEC:
         Process_Command_GetDistance(SONAR_COMMAND_USEC);
         break;
      default:
         Send_NACK();
         break;
   }
   return(0);
}

int main(void)
{
   unsigned int Command;

   Init();

   while (1)
   {
      if (UART_Data())
      {
         Command = UART_ReceiveWord();
         Process_Command(Command);
      } /* if UART_Data */
   }
}

