#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include <string.h>

/* unsigned int PodType; */
#define PodType POD_TYPE_THERMOMETER
#define POD_VAR_URL "http://pel.cs.byu.edu/~sorenson/Thesis/PODs/Thermometer/"

#include "PODDefines.h"
#include "PODIncludes.h"
#include "Serial.h"
#include "Sleep.h"
#include "ADC.h"

//char CompileDate __attribute__ ((section (".eeprom"))) = __DATE__;

/* Local POD Variables (eeprom addresses, lengths, etc.) */
#define POD_VAR_UNITS_ADDR     0x0040
#define POD_VAR_UNITS_LEN      0x0001

/* DisplayUnits - Units to display temperature in */
unsigned char DisplayUnits;

/* read variables from the eeprom */
void ReadVars()
{
   DisplayUnits = eeprom_read_byte((uint8_t *)(POD_VAR_UNITS_ADDR));
   if (DisplayUnits != 'C') DisplayUnits = 'F';
}

int Init()
{
   int Prescaler;

   cli();
   Prescaler = 5;
   UART_Init(23);
   SetupADC(0, 1);

   ReadVars();

   sei();
   return(0);
}

/* returns the temperature in 100ths F */
unsigned long GetReading()
{
   unsigned long ADCValue;
   unsigned long ADCAccumulation;
   int Counter;

   ADCAccumulation = 0;
   for (Counter = 0 ; Counter < 16 ; Counter ++)
   {
      ADCValue = StartADC(0);
      ADCAccumulation += ADCValue;
   }
   ADCValue = ADCAccumulation / 16;

   ADCValue = ADCValue * 25;
   return(ADCValue);
}

/* Assumes temperature is in 100ths C */
unsigned long CtoF(unsigned long InputTemp)
{
   unsigned long ReturnValue;
   ReturnValue = ((InputTemp * 9) / 5) + 3200;
   return(ReturnValue);
}

/* Assumes temperature is in 100ths F */
unsigned long FtoC(unsigned long InputTemp)
{
   unsigned long ReturnValue;
   ReturnValue = ((InputTemp - 3200) * 5) / 9;
   return(ReturnValue);
}

/* Assumes temperature coming in as 100ths F */
unsigned long TemperatureConvert(unsigned long InputTemp, char Unit)
{
   if (Unit == 'F') return(InputTemp);
   if (Unit == 'C') return(FtoC(InputTemp));
   return(0);
}

void Process_Command_GetTemp()
{
   unsigned long TemperatureReading; /* Temperature reading from GetReading (in F) */
   unsigned long CurrentTemperature; /* CurrentTemperature in appropriate units */

   TemperatureReading = GetReading();
   CurrentTemperature = TemperatureConvert(TemperatureReading, DisplayUnits);
//   CurrentTemperature = 1060;
   UART_TransmitByte(POD_RESPONSE_ACK);
   UART_TransmitWord(CurrentTemperature / 100);
   UART_TransmitByte(CurrentTemperature % 100); /* send hundredths */
}

void Process_Command_GetUnits()
{
   UART_TransmitByte(POD_RESPONSE_ACK);
   if (DisplayUnits == 'F') UART_TransmitByte('F');
   else if (DisplayUnits == 'C') UART_TransmitByte('C');
   else UART_TransmitByte('*');
}
void Process_Command_SetUnits()
{
   unsigned char Units;
   Units = UART_ReceiveByte();
   if (Units == 'C') DisplayUnits = 'C';
   else DisplayUnits = 'F';
   eeprom_write_byte((uint8_t *)(POD_VAR_UNITS_ADDR), DisplayUnits);
   UART_TransmitByte(POD_RESPONSE_ACK);
}

int Process_Command(unsigned int Command)
{
   switch (Command)
   {
      case POD_COMMAND_PING:
         Process_Command_Ping();
         break;
      case POD_COMMAND_GETTYPE:
         Process_Command_GetType();
         break;
      case POD_COMMAND_GETINFO:
         Process_Command_GetInfo();
         break;
      case POD_COMMAND_RESET:
/*         Process_Command_Reset(); */
         break;
      case POD_COMMAND_GETNAME:
         Process_Command_GetName();
         break;
      case POD_COMMAND_SETNAME:
         Process_Command_SetName();
         break;
      case POD_COMMAND_THERMOMETER_GET_TEMP:
         Process_Command_GetTemp();
         break;
      case POD_COMMAND_THERMOMETER_GET_UNITS:
         Process_Command_GetUnits();
         break;
      case POD_COMMAND_THERMOMETER_SET_UNITS:
         Process_Command_SetUnits();
         break;
      default:
         Send_NACK();
         break;
   }
   return(0);
}

int main(int argc, char *argv[])
{
   unsigned long ADCValue;
   unsigned int Command;

   Init();

   ADCValue = StartADC(0);

   ADCValue = GetReading();
   while (1)
   {
      if (UART_Data()) /* some data is waiting */
      {
         Command = UART_ReceiveWord();
         Process_Command(Command);
      } /* if UART_Data */
   } /* end while */
   return(0);
}
